{$APPTYPE CONSOLE}

uses
  SysUtils;

type
  TNumber = 0..15;
  TMatrix = array[1..7] of array[1..5] of char;

var fin, fout, fans: text;
  
  
procedure Ex(const s: string; ec: integer);
begin
  write(s);
  try
    close(fin); close(fout); close(fans);
  except
  end;
  halt(ec);
end;

const
  numbers: array[TNumber] of TMatrix = (
  (
    '.###.',
    '#...#',
    '#..##',
    '#.#.#',
    '##..#',
    '#...#',
    '.###.'
  ),
  (
    '...##',
    '..#.#',
    '.#..#',
    '#...#',
    '....#',
    '....#',
    '....#'
  ),
  (
    '.###.',
    '#...#',
    '....#',
    '...#.',
    '..#..',
    '.#...',
    '#####'
  ),
  (
    '.###.',
    '#...#',
    '....#',
    '.###.',
    '....#',
    '#...#',
    '.###.'
  ),
  (
    '#...#',
    '#...#',
    '#...#',
    '#...#',
    '.####',
    '....#',
    '....#'
  ),
  (
    '#####',
    '#....',
    '#....',
    '####.',
    '....#',
    '#...#',
    '.###.'
  ),
  (
    '.###.',
    '#...#',
    '#....',
    '####.',
    '#...#',
    '#...#',
    '.###.'
  ),
  (
    '#####',
    '....#',
    '....#',
    '...#.',
    '..#..',
    '.#...',
    '#....'
  ),
  (
    '.###.',
    '#...#',
    '#...#',
    '.###.',
    '#...#',
    '#...#',
    '.###.'
  ),
  (
    '.###.',
    '#...#',
    '#...#',
    '.####',
    '....#',
    '#...#',
    '.###.'
  ),
  (
    '..#..',
    '.#.#.',
    '#...#',
    '#...#',
    '#####',
    '#...#',
    '#...#'
  ),
  (
    '####.',
    '#...#',
    '#...#',
    '####.',
    '#...#',
    '#...#',
    '####.'
  ),
  (
    '.###.',
    '#...#',
    '#....',
    '#....',
    '#....',
    '#...#',
    '.###.'
  ),
  (
    '###..',
    '#..#.',
    '#...#',
    '#...#',
    '#...#',
    '#..#.',
    '###..'
  ),
  (
    '#####',
    '#....',
    '#....',
    '####.',
    '#....',
    '#....',
    '#####'
  ),
  (
    '#####',
    '#....',
    '#....',
    '####.',
    '#....',
    '#....',
    '#....'
  )
  );
  space = 1;
  MaxQ = 16*16;
  MaxN = 40;
  MaxLength = 6*MaxN-1;

var symbols: array[0..MaxQ] of TMatrix;
    s, ss, s1, s2: string;
    i, j, l: integer;

function ApplySymbols(a, b: integer): TMatrix;
var i, j: integer;
begin
  Result := numbers[a];
  for i := 1 to 7 do
    for j := 1 to 5 do
      if Result[i, j] = '.' then Result[i, j] := numbers[b, i, j];
end;

function GetNum(c: char): integer;
begin
  if c in ['0'..'9'] then Result := ord(c) - ord('0') else Result := ord(c) - ord('A') + 10;
end;        

procedure CheckString(s: string);
var i: integer;
begin
  if s = 'IMPOSSIBLE' then exit;
  for i := 1 to length(s) do
    if not (s[i] in ['0'..'9', 'A'..'F']) then
      Ex(format('Invalid character ''%s''(code: %d) in %d position', [s[i], ord(s[i]), i]), 2);
end;

function MatrixEqual(a, b: integer): boolean;
var i, j: integer;
begin
  Result := true;
  for i := 1 to 7 do
    for j := 1 to 5 do
      if symbols[a, i, j] <> symbols[b, i, j] then
        begin
          Result := false;
          exit;
        end;
end;

begin
  for i := 0 to 15 do
    for j := i to 15 do
      begin
        symbols[16*i+j] := ApplySymbols(i, j);
        symbols[16*j+i] := symbols[16*i+j];
      end;
  try
    assign(fin,paramstr(1)); reset(fin);
  except on e: Exception do
    Ex('Error opening input: ' + e.Message, 3);
  end;
  try
    assign(fans,paramstr(3)); reset(fans);   
  except on e: Exception do
    Ex('Error opening answer: ' + e.Message, 3);
  end;
  try
    assign(fout,paramstr(2)); reset(fout);
  except on e: Exception do
    Ex('Can''t open output: ' + e.Message, 2);
  end;
  try
    readln(fans, s);     
    read(fans, ss);
  except on e: Exception do
    Ex('Answer writting error: ' + e.Message, 3);
  end;
  try
    if SeekEOF(fout) then Ex('There''s no data in output', 2);
    readln(fout, s1); CheckString(s1);
    if s <> 'IMPOSSIBLE' then
      begin
        if s1 = 'IMPOSSIBLE' then Ex('Two strings expected but ''IMPOSSIBLE'' found', 1);
        if SeekEOF(fout) then Ex('Too few strings in output', 2);
        readln(fout, s2); CheckString(s2);
      end
    else
      begin
        if (s1 <> s) then Ex('''IMPOSSIBLE'' expected but other strings found', 1) else Ex('', 0);
      end;
    if not SeekEOF(fout) then Ex('There''s extra data in output', 2);
  except on e: Exception do
    Ex('Output writting error: ' + e.Message, 2);
  end;
  l := length(s);
  if (length(s1) <> length(s2)) or (length(s1) <> l) then
    Ex(format('Invalid strings lengths: (first: %d; second: %d). Answer length is %d', [length(s1), length(s2), l]), 2);
  for i := 1 to l do
    begin
      if not MatrixEqual(GetNum(s1[i])*16+GetNum(s2[i]), GetNum(s[i])*16+GetNum(ss[i])) then
        Ex(format('''%s'' and ''%s'' instead of ''%s'' and ''%s'' in %d position', [s1[i], s2[i], s[i], ss[i], i]), 1);
    end;
  Ex('', 0);
end.
